
/** INCLUDES *******************************************************/
#include "mscext.h"
#include "USB/usb.h"
#include "USB/usb_function_generic.h"

#include "HardwareProfile.h"
#include "user.h"

#include "DAC.h"
#include "Fpga.h"

/** V A R I A B L E S ********************************************************/

#if defined(__18F2455) || defined(__18F2550) || defined(__18F4455) || defined(__18F4550)\
    || defined(__18F2458) || defined(__18F2453) || defined(__18F4558) || defined(__18F4553)
    #pragma udata USB_VARIABLES=0x500
#elif defined(__18F4450) || defined(__18F2450)
    #pragma udata USB_VARIABLES=0x480
#else
    #pragma udata
#endif

DATA_PACKET INPacket;
DATA_PACKET OUTPacket;
#pragma udata

USB_HANDLE USBGenericOutHandle = 0;
USB_HANDLE USBGenericInHandle = 0;

BOOL blinkStatusValid = TRUE;
BYTE counter;


/** P R I V A T E  P R O T O T Y P E S ***************************************/

void BlinkUSBStatus(void);
void ServiceRequests(void);

/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
	INTCONbits.GIE = 0;
    RCONbits.IPEN = 0;			// No int priority
    INTCONbits.PEIE = 1;		// Peripheral INT enable

    mInitAllLEDs();
	InitDAC();
	InitFpga();
      
	INTCONbits.GIE = 1;			// enable global

    blinkStatusValid = TRUE;
}//end UserInit


// * Overview:        This function is a place holder for other user routines.
// *                  It is a mixture of both USB and non-USB tasks.
void ProcessIO(void)
{   
    //Blink the LEDs according to the USB device status
    if(blinkStatusValid)
    {
        BlinkUSBStatus();
    }

    // User Application USB tasks
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;

    //respond to any USB commands that might have come over the bus
    ServiceRequests();

}//end ProcessIO


// * Overview:        This function takes in the commands from the PC from the
// *                  application and executes the commands requested
void ServiceRequests(void)
{
    BYTE index;
	BYTE_VAL tmp;
	DWORD_VAL freq;
	WORD_VAL phase;
	WORD_VAL daval;
    
    //Check to see if data has arrived
    if(!USBHandleBusy(USBGenericOutHandle))
    {        
        //if the handle is no longer busy then the last
        //transmission is complete
       
        counter = 0;

        INPacket.DP.CMD=OUTPacket.DP.CMD;
        INPacket.DP.Len=OUTPacket.DP.Len;		

        //process the command
        switch(OUTPacket.DP.CMD)
        {
            case READ_VERSION:
                //dataPacket._byte[1] is len
                INPacket._byte[2] = MINOR_VERSION;
                INPacket._byte[3] = MAJOR_VERSION;
                counter=0x04;
                break;
                        
			case INITWRITE:
				FpgaInitWrite();
				INPacket.DP.Len = 1;
				INPacket.DP.Data[0] = 0x00;
				counter = 3;
				break;

			case WRITERAM:
				FpgaWrite(OUTPacket.DP.Data, OUTPacket.DP.Len);	// write data to ram
				INPacket.DP.Len = 1;
				INPacket.DP.Data[0] = 0x00;
				counter = 3;
				break;

			case STARTGEN:
				freq.byte.LB = OUTPacket.DP.Data[0];
				freq.byte.HB = OUTPacket.DP.Data[1];
				freq.byte.UB = OUTPacket.DP.Data[2];
				freq.byte.MB = OUTPacket.DP.Data[3];
				phase.byte.LB = OUTPacket.DP.Data[4];
				phase.byte.HB = OUTPacket.DP.Data[5];
				FpgaInitGenerate(freq.Val, phase.Val, OUTPacket.DP.Data[6]);
				INPacket.DP.Len = 1;
				INPacket.DP.Data[0] = 0x00;
				counter = 3;
				break;

			case MANCLK:									// manual clock. data not interpreted.
				ManualClk();
				INPacket.DP.Len = 1;
				INPacket.DP.Data[0] = 0x00;
				counter = 3;
				break;

			case FPGAHARDRESET:
				FpgaResetStop();
				INPacket.DP.Len = 1;
				INPacket.DP.Data[0] = 0x00;
				counter = 3;
				break;

			case SETDAC:
				daval.byte.HB = OUTPacket.DP.Data[1];
				daval.byte.LB = OUTPacket.DP.Data[2];
				SetDAC(OUTPacket.DP.Data[0], daval);
				INPacket.DP.Len = 1;
				INPacket.DP.Data[0] = 0x00;
				counter = 3;
				break;


            case RESET:
#ifdef __18CXX
                Reset();
#endif
                break;
                
            default:
#ifdef __18CXX
                Nop();
#endif
                break;
        }//end switch()
        if(counter != 0)
        {
            if(!USBHandleBusy(USBGenericInHandle))
            {
                USBGenericInHandle = USBGenWrite(USBGEN_EP_NUM,(BYTE*)&INPacket,counter);
            }
        }//end if
        
        //Re-arm the OUT endpoint for the next packet
        USBGenericOutHandle = USBGenRead(USBGEN_EP_NUM,(BYTE*)&OUTPacket,USBGEN_EP_SIZE);
    }//end if

}//end ServiceRequests


// * Overview:        BlinkUSBStatus turns on and off LEDs 
// *                  corresponding to the USB device state.
void BlinkUSBStatus(void)
{
    static WORD led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    if(USBSuspendControl == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            if(mGetLED_1())
            {
                mLED_2_On();
            }
            else
            {
                mLED_2_Off();
            }
        }//end if
    }
    else
    {
        if(USBDeviceState == DETACHED_STATE)
        {
            //mLED_Both_Off();
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(USBDeviceState == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(USBDeviceState == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(USBDeviceState == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(USBDeviceState == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_2_Toggle();
                mLED_1_Off();
            }//end if
        }
        else if(USBDeviceState == CONFIGURED_STATE)
        {
            if(led_count==0)
            {      
                mLED_1_Toggle();         
                if(mGetLED_1())
                {
                    mLED_2_Off();
                }
                else
                {
                    mLED_2_On();
                }
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus



/** EOF user.c ***************************************************************/
